// Include le Librerie per il Server WiFi
#include <ESP8266WiFi.h>                              
#include <WiFiClient.h> 
#include <ESP8266WebServer.h>

// Include le Librerie per il sensore IR
#include <IRremote.h>

// Definire Motori su Modulo L298N
#define ENA   D8    // Motore Destra  
#define IN_1  D7    // In1 motore Destra       
#define IN_2  D6    // In2 motore Destra      
#define IN_3  D5    // In3 motore Sinistra        
#define IN_4  D4    // In4 motore Sinistra       
#define ENB   D3    // Motore Sinistra    

// Imposta il pin del cicalino
byte buzzer = D2;

// Imposta il pin del ricevitore IR 
const int pinRicevitoreIR = D1;      // pin a cui  collegato il ricevitore
IRrecv irrecv(pinRicevitoreIR);      
decode_results risultato;            // salva il risultato ricevuto dal rilevatore IR

//String per lo stato di comando
String command;       

// Impostare velocita' motori
const int velocita = 1023;   
const int ridotta = 0;

// Creazione di un server web in ascolto sulla porta 80
const char* ssid = "Rover WiFI";
ESP8266WebServer server(80);


void setup() {
 

Serial.begin(115200);    // inizializza Serial monitor
irrecv.enableIRIn();     // Attiva l'oggetto del ricevitore

//Definire pin motori
pinMode(ENA,  OUTPUT); 
pinMode(IN_1, OUTPUT);
pinMode(IN_2, OUTPUT);
pinMode(IN_3, OUTPUT);
pinMode(IN_4, OUTPUT);
pinMode(ENB,  OUTPUT); 

//Definire pin motori
pinMode(buzzer, OUTPUT);

// Connessione WiFi
WiFi.mode(WIFI_AP);
WiFi.softAP(ssid);

IPAddress myIP = WiFi.softAPIP();
Serial.print("AP IP address: ");
Serial.println(myIP);

// Starting WEB-server
server.on ( "/", HTTP_handleRoot );
server.onNotFound ( HTTP_handleRoot );
server.begin();
}

//Impostazioni server 
void HTTP_handleRoot(void) {
if( server.hasArg("State") ){
Serial.println(server.arg("State"));
}
server.send ( 200, "text/html", "" );
delay(1);
}

// Impostazioni per i motori
void Indietro(){ 
digitalWrite(IN_1, HIGH);
digitalWrite(IN_2, LOW);
analogWrite (ENA, velocita);

digitalWrite(IN_3, HIGH);
digitalWrite(IN_4, LOW);
analogWrite (ENB, velocita);
}

void IndietroDestra(){ 
digitalWrite(IN_1, HIGH);
digitalWrite(IN_2, LOW);
analogWrite(ENA, ridotta);

digitalWrite(IN_3, HIGH);
digitalWrite(IN_4, LOW);
analogWrite(ENB, velocita);
}

void IndietroSinistra(){ 
digitalWrite(IN_1, HIGH);
digitalWrite(IN_2, LOW);
analogWrite(ENA, velocita);

digitalWrite(IN_3, HIGH);
digitalWrite(IN_4, LOW);
analogWrite(ENB, ridotta);
}

void Avanti(){ 
digitalWrite(IN_1, LOW);
digitalWrite(IN_2, HIGH);
analogWrite (ENA, velocita);

digitalWrite(IN_3, LOW);
digitalWrite(IN_4, HIGH);
analogWrite (ENB, velocita);
}

void AvantiDestra(){
digitalWrite(IN_1, LOW);
digitalWrite(IN_2, HIGH);
analogWrite(ENA, ridotta);
 
digitalWrite(IN_3, LOW);
digitalWrite(IN_4, HIGH);
analogWrite(ENB, velocita);
}

void AvantiSinistra(){
digitalWrite(IN_1, LOW);
digitalWrite(IN_2, HIGH);
analogWrite(ENA, velocita);

digitalWrite(IN_3, LOW);
digitalWrite(IN_4, HIGH);
analogWrite(ENB, ridotta);
}

void Destra(){ 
digitalWrite(IN_1, HIGH);
digitalWrite(IN_2, LOW);
analogWrite (ENA, velocita);

digitalWrite(IN_3, LOW);
digitalWrite(IN_4, HIGH);
analogWrite (ENB, velocita);
}

void Sinistra(){
digitalWrite(IN_1, LOW);
digitalWrite(IN_2, HIGH);
analogWrite (ENA, velocita);

digitalWrite(IN_3, HIGH);
digitalWrite(IN_4, LOW);
analogWrite (ENB, velocita);
}

void Fermo(){  
digitalWrite(IN_1, LOW);
digitalWrite(IN_2, LOW);
analogWrite(ENA, velocita);

digitalWrite(IN_3, LOW);
digitalWrite(IN_4, LOW);
analogWrite(ENB, velocita);
}

//Imposta il movimento casuale nella modalita' gioco
void Gioco(){
switch(random(9)) {
case 0:
delay(1000);
case 1:
Avanti();
break;
case 2:
Indietro();
break;
case 3:
Destra();
break;
case 4:
Sinistra();
break;
case 5:
AvantiDestra();
break;
case 6:
AvantiSinistra();
break;
case 7:
IndietroDestra();
break;
case 8:
IndietroSinistra();
break;
}
}

//Impostazioni per i suoni
void Blaster01(){
for(int i = 0; i <300; i++)
{digitalWrite(buzzer,HIGH);
delayMicroseconds(i);
digitalWrite(buzzer,LOW);
delayMicroseconds(i);}
{for(int o = 0; o <500; o++)
{digitalWrite(buzzer,HIGH);
delayMicroseconds(o);
digitalWrite(buzzer,LOW);
delayMicroseconds(o);}}
}

void Blaster02()
{for(int x = 0; x <100; x++){
digitalWrite(buzzer,HIGH);
delayMicroseconds(x);
digitalWrite(buzzer,LOW);
delayMicroseconds(x);}
}

void Blaster03()
{digitalWrite(buzzer,HIGH);
delayMicroseconds(150);
digitalWrite(buzzer,LOW);
delayMicroseconds(150);}

void Blaster04(float num,float period) {
int fxdelay=num+num*sin(millis()/period);
for(int i=0;i<5;i++){
digitalWrite(buzzer,HIGH);
delayMicroseconds(fxdelay);
digitalWrite(buzzer,LOW);
delayMicroseconds(fxdelay);
} 
}

void Pausa(){}


void loop() {


server.handleClient();
command = server.arg("State");

//comandi per i motori
if      (command == "A") Avanti();
else if (command == "I") Indietro();
else if (command == "D") Destra();
else if (command == "S") Sinistra();
else if (command == "AD") AvantiDestra();
else if (command == "AS") AvantiSinistra();
else if (command == "ID") IndietroDestra();
else if (command == "IS") IndietroSinistra();
else if (command == "F") Fermo();
else if (command == "G") Gioco();

//Comandi per i suoni
else if (command == "PX") Blaster01();
else if (command == "PY") Blaster02();
else if (command == "PA") Blaster03();
else if (command == "PB") Blaster04(200,29);
else if (command == "P") Pausa();

//Impostazioni per ricezione segnale IR nella modalita' gioco 
if (irrecv.decode(&risultato)) {
Serial.println("Ricevuto = ");
Serial.print(risultato.value,HEX);

switch (risultato.value) {
case 0xCDB54565:
Blaster01();
Fermo();
delay(10000);
Blaster01();
break;
}

irrecv.resume();
}
}
